function [lmax, X1_t, X2_t, X3_t, H] = boxcox_roc3(X1, X2, X3)
    % Find maximum of the profile log likelihood function by finding the
    %  minimum of the negative profile log likelihood
    p = @(lambda) -pLogLik(X1, X2, X3, lambda); 
    options=optimset('TolX',1e-10);%, 'MaxIter', 20000, 'MaxFunEvals', 20000);
    [lmax, pmax, exitflag] = fminsearch(p, 1, options); 
    loglikmax = -pmax;
    
    % Transform XD and XH
    X1_t = xlambda(X1, lmax);
    X2_t = xlambda(X2, lmax);
    X3_t = xlambda(X3, lmax);
    
    % Plot the profile log likelihood function
%     L = [(lmax-1):0.01:(lmax+1)];
%     plot(L, -p(L));
%     hold on;
%     line([lmax lmax], [-p(lmax-1) loglikmax]);
    
    % Calculate Hessian Matrix of the Transformation
    n1 = max(size(X1)); n2 = max(size(X2)); n3 = max(size(X3));
    m1 = mean(X1_t); m2 = mean(X2_t); m3 = mean(X3_t);
    s1 = std(X1_t,1); s2 = std(X2_t,1); s3 = std(X3_t,1);
    Lambda = lmax;
    d2L_dm12 = -n1/s1^2;
    d2L_ds12 = n1/s1^2 - 3*sum((X1_t - m1).^2)/s1^4;
    d2L_dm22 = -n2/s2^2;
    d2L_ds22 = n2/s2^2 - 3*sum((X2_t - m2).^2)/s2^4;
    d2L_dm32 = -n3/s3^2;
    d2L_ds32 = n3/s3^2 - 3*sum((X3_t - m3).^2)/s3^4;

    d2L_dm1dLmb = 1/s1.^2*sum((X1.^Lambda.*log(X1)./Lambda...
                    - (X1.^Lambda - 1)/Lambda^2));
    d2L_ds1dLmb = 2/s1.^3*sum((X1_t-m1).*((X1.^Lambda.*log(X1))./Lambda...
                    - (X1.^Lambda - 1)./Lambda^2));
    d2L_dm2dLmb = 1/s2.^2*sum((X2.^Lambda.*log(X2)./Lambda...
                    - (X2.^Lambda - 1)./Lambda^2));
    d2L_ds2dLmb = 2/s2.^3*sum((X2_t-m2).*((X2.^Lambda.*log(X2))/Lambda...
                    - (X2.^Lambda - 1)./Lambda^2));
    d2L_dm3dLmb = 1/s3.^2*sum((X3.^Lambda.*log(X3)./Lambda...
                    - (X3.^Lambda - 1)./Lambda^2));
    d2L_ds3dLmb = 2/s3.^3*sum((X3_t-m3).*((X3.^Lambda.*log(X3))/Lambda...
                    - (X3.^Lambda - 1)./Lambda^2));

    dx_dLmb = @(x) (x.^Lambda.*log(x))./Lambda - (x.^Lambda - 1)./Lambda^2;
    d2x_dLmb2 = @(x) (2*(x.^Lambda - 1))./Lambda^3 - ....
        (2.*x.^Lambda.*log(x))/Lambda.^2 + (x.^Lambda.*log(x).^2)./Lambda;
   
    d2L_dLmb2 = -1/s1^2*sum(dx_dLmb(X1).^2 + X1_t.*d2x_dLmb2(X1))...
                    + m1/s1^2*sum(d2x_dLmb2(X1)) ...
                -1/s2^2*sum(dx_dLmb(X2).^2 + X2_t.*d2x_dLmb2(X2))...
                    + m2/s2^2*sum(d2x_dLmb2(X2)) ...
                -1/s3^2*sum(dx_dLmb(X3).^2 + X3_t.*d2x_dLmb2(X3))...
                    + m3/s3^2*sum(d2x_dLmb2(X3));

    H = [d2L_dm12  0 0 0 0 0 d2L_dm1dLmb; ...
        0 d2L_ds12   0 0 0 0 d2L_ds1dLmb; ...
        0 0 d2L_dm22   0 0 0 d2L_dm2dLmb; ...
        0 0 0 d2L_ds22   0 0 d2L_ds2dLmb; ...
        0 0 0 0 d2L_dm32   0 d2L_dm3dLmb; ...
        0 0 0 0 0 d2L_ds32   d2L_ds3dLmb; ...
        d2L_dm1dLmb d2L_ds1dLmb d2L_dm2dLmb d2L_ds2dLmb...
                    d2L_dm3dLmb d2L_ds3dLmb d2L_dLmb2 ];
end

% Profile Log-Likelihood
function r = pLogLik(X1, X2, X3, L)
    % Calculate X^(Lambda) for each population
    LX1 = xlambda(X1,L);
    LX2 = xlambda(X2,L);
    LX3 = xlambda(X3,L);
    
    % Calculate Maximum Likelihood Estimates of sigma
    n1 = max(size(X1));
    n2 = max(size(X2));
    n3 = max(size(X3));
    s1 = std(LX1,1);
    s2 = std(LX2,1);
    s3 = std(LX3,1);
    
    % Calculate Profile Log-Likelihood 
    r = -n1.*log(s1) + (L-1).*sum(log(X1)) ...
        -n2.*log(s2) + (L-1).*sum(log(X2)) ...
        -n3.*log(s3) + (L-1).*sum(log(X3));
        
end

function r = xlambda(x, lmb)
    if abs(lmb) < 1e-10
        r = log(x);
    else
        r = (x.^lmb-1)./lmb;
    end
end

